@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
+ * Copyright (c) 2000, 2007 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -11,17 +11,16 @@
 package org.aspectj.org.eclipse.jdt.internal.compiler.ast;
 
 import org.aspectj.org.eclipse.jdt.internal.compiler.ASTVisitor;
+import org.aspectj.org.eclipse.jdt.internal.compiler.classfmt.ClassFileConstants;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.*;
 
 public class ImportReference extends ASTNode {
 
 	public char[][] tokens;
 	public long[] sourcePositions; //each entry is using the code : (start<<32) + end
-	public boolean onDemand = true; //most of the time
 	public int declarationEnd; // doesn't include an potential trailing comment
 	public int declarationSourceStart;
 	public int declarationSourceEnd;
-	public boolean used;
 	public int modifiers; // 1.5 addition for static imports
 	public Annotation[] annotations;
 
@@ -33,14 +32,16 @@ public class ImportReference extends ASTNode {
 
 		this.tokens = tokens;
 		this.sourcePositions = sourcePositions;
-		this.onDemand = onDemand;
+		if (onDemand) {
+			this.bits |= ASTNode.OnDemand;
+		}
 		this.sourceEnd = (int) (sourcePositions[sourcePositions.length-1] & 0x00000000FFFFFFFF);
 		this.sourceStart = (int) (sourcePositions[0] >>> 32);
 		this.modifiers = modifiers;
 	}
 	
 	public boolean isStatic() {
-		return (this.modifiers & AccStatic) != 0;
+		return (this.modifiers & ClassFileConstants.AccStatic) != 0;
 	}
 
 	/**
@@ -63,20 +64,15 @@ public class ImportReference extends ASTNode {
 			if (i > 0) output.append('.');
 			output.append(tokens[i]);
 		}
-		if (withOnDemand && onDemand) {
+		if (withOnDemand && ((this.bits & ASTNode.OnDemand) != 0)) {
 			output.append(".*"); //$NON-NLS-1$
 		}
 		return output;
 	}
 
 	public void traverse(ASTVisitor visitor, CompilationUnitScope scope) {
-
+		// annotations are traversed during the compilation unit traversal using a class scope
 		visitor.visit(this, scope);
-		if (this.annotations != null) {
-			int annotationsLength = this.annotations.length;
-			for (int i = 0; i < annotationsLength; i++)
-				this.annotations[i].traverse(visitor, scope);
-		}
 		visitor.endVisit(this, scope);
 	}
 }
